//******************************************************************************
//   eRIC eASYRadio Application
//   Low Power Radio Solutions
//   January 2015
//   Built with CCS V6.0.1.00040
//	 Both eRIC9 and eRIC4 use this file. #defines below
//******************************************************************************



/* Version History
Version 1.0:    Module type either 9 or 4 is added after eRIC when T3 command is used, based on serial number
Version 1.1:    Temperature problem is fixed and also WOR feature added. UartRXinterrupt flag bug fixed(Unexpected data on Uart RX is handled)
                1.1Beta Swap 869.75 or 915 based on pin14 as #define SwapFrequency_Pin14
                Demo1.1 is added in to this app
                Global variable function is added
Version 1.2:    Added Software Carrier Detect into the easyRadio() function
Version 1.3:    Added Timer0, PWM definitions and Motiv8 workshop project.
Version 1.4:    Added AES, AES command is explained below:
                ER_CMD#A1xy...16y where A1 is AES command and x is 0/1/2/3 where 0 AES is Off,1 AES is On with both encryption decryption. 2 AES ON with only encryption, 3 AES on with only decryption.
                I2C DEFINITIONS are added: Condition for I2C function is CpuFrequency>=I2CClock*5
                For I2C{The first byte after a START condition consists of a 7-bit slave address and the R/W bit. When R/W = 0,
                the master transmits data to a slave. When R/W = 1, the master receives data from a slave. The ACK bit
                is sent from the receiver after each byte on the ninth SCL clock.}
                Uncalibrated modulation is added ER_CMD#T1U
                Bug regarding Carrier detect when D7 is on is resolved by simply not implementing carrier detect when D is greater than 0
Version 1.5:    CPU frequency can be changed using commands.SWT2 held high on reset will reset Eeprom to default.Power on reset R1 command is added.
                AES command modified. SwapFrequency functionality is modified(bug resolved).
                Low power commands are added ER_CMD#A2x .
                Software modified for FCC. Power can be changed when EEprom location 20 -29 is changed (for FCC only,as it needs removing #define limitOnPower_FCC)
                Handshaking command added
                Only CPULPM command added. ER_CMD#A8x
Version 1.5.1:  A bug fix of Global interrupt not enabled in demo mode.Fixed it by adding global interrupt.
                delays in demo mode are using cycles which are changed to eRIC_Delay() in this version.
Version 1.5.2:  1.5.1.a and 1.5.2 are same
Version 1.5.3:  Pin1 power down/wake module. Only for specific customer. Do NOT USE FOR OTHERS
Version 1.5.4:  Eeprom writings is not allowed in D1-7. UARTABUSY is added. Uart interrupt vector modified.ER_CMD#R0 modified.
                Command timeout added to delay Rx traffic when command is sent.
                Handshaking is modified. Before rx data was flushed when busy, now it doesn't flush but rather it doesn't deliver rx data
                SetLPM is modified when already in D1-7.
                eRIC_RfDataReceivedInterrupt(); is added,, this is called when radio received any data..all code should be in this.
                Fixed bug to read ADC, as it drawing more current in low power modes
Version 1.5.5:  In LPM2 modes, when Radio receives data and needs to deliver through UART, the clock takes time to reach the desired speed and delivers garbage if it is not at right speed
                Fixed Garbage Uart delivery in LPM2 by Checking ACLK timing with a Pin toggle which is based on SMCLK and waiting until Pin toggling is close enough to be at LPM2
                Variable which are float are modified to long to process the calculation quicker. Ledtimertimeout is modified. Instead of calculation in timer, a fixed pre calculated is loaded which reduces processor time.
                Carrier Detect command added: ER_CMD#a1pxx p = polarity: 0 = rest at 0 (1 when carrier detect)
                                                                         1 = idle high, (0 when carrier detect)
                                                           xx = RSSI value in ASCII HEX.To disable, set xx = FF

                ER_CMD#R0 command is modified so that it does POR at the end of the command.
                Command Timeout is modified: A received data is not delivered for a timeout delay of 200ms if a command is sent and is not completed. But a timeout delay of 20bits delay is added if command is completed before delivering data
                Cause of last reset can be retrieved using ER_CMD#R? command. Refer page 78 of slau259e.pdf document
*/



/* Eeprom location and data being held in that location
 * Eeprom address 0 for reseting radio parameters to default if the location contains data less than 2
 * Eeprom address 1 contains baud rate data
 * Eeprom address 2 contains channel data
 *              THis is removed/modified in V1.5 Eeprom address 3 contains power data
 * Eeprom address 3 contains powerlevel
 * Eeprom address 4 contains Uart baud rate data
 * Eeprom address 5 contains Rx powerlevel data used in low power mode
 * Eeprom address 6 contains Tx powerlevel data used in low power mode
 * Eeprom address 7,8,9,10 contains frequency data which is 4 bytes in hex
 * Eeprom address 11,12 contains Group ID which is 2 bytes
 * Eeprom address 13 contain AES whether enabled or disable. if 0 it is disabled or 1 for enabled
 *              This is removed in V1.5 Eeprom address 14 contains AES mode, 0 for encryption, 1 for decryption, 2Generate key may not used, 3 for to use generated Key
 *              This is removed in V1.5 Eeprom address 15-31 (16bytes) contains AES KEY. V1.4 // storing AES key here is removed for V4.1 and V1.5.Its been stored in discrete location
 * Eeprom address 14,15,16,17 contains CPUfrequency data which is 4 bytes in hex .Added in V1.5
 * Eeprom address 18 contains LPM level settings which is a char V1.5
 * Eeprom address 19 contains Carrier settings to start module with, this is added for FCC testing purpose  V1.5
 * Eeprom address 20 - 29 contains the 10 data values for power settings V1.5
 * Eeprom address 30 contains handshaking On/Off
 * Eeprom address 31 contains CPULPM level settings which is a char V1.5
 * Eeprom address 32 contains CurrentCarrierDetectRSSI which is int in V1.5.5. 32 char has polarity. CurrentCarrierDetectRSSI
 * Eeprom address 33 contains CurrentCarrierDetectRSSI which is int in V1.5.5. 33 char has RSSI. CurrentCarrierDetectRSSI  ER_CMD#a1pxx
 * */



#include<cc430f5137.h>
#include "eRIC.h"
#include <stdio.h>
#include <string.h>

//Power is limited to -3dbm and 7dbm max at 915 and 868Mhz
#define limitOnPower_FCC

// Choose module type to ensure correct frequency selections
//#define eRIC4
#define	eRIC9


#ifdef eRIC4
#define ModuleFreq 434000000
#else
#define ModuleFreq 869750000
#endif

#define Pin2_CTS() Pin2_FunctionUartABusy();
#define Pin5_RTS() {Pin5_SetAsInput();Pin5_PullDownEnable();}

#if(ModuleFreq == 869750000)
#define SwapFrequency_Pin14
#endif


void easyRadio();
void Demo_Mode();
void SetupTimer(); //Added V1.5
void Radio_Stored_settings();
void Initialise_Globalvariables();
int ER_Command(char *buff , unsigned char count);

void SetUARTTimeout();
void SetLedtimerTimeOut();//V1.5
void SetCarrierModeOnStartup(unsigned char Mode); //V1.5 For FCC purpose
void Power(); //V1.5

volatile unsigned long Uart_buff_Timeout;
volatile unsigned long Uart_Timeout;
//volatile unsigned char UART_RX_Buffer[255];
volatile unsigned char UART_RX_Buffer[256]; //changed from above in V1.4 while doing AES

const char VersionString[] = "eRIC  V1.5.5\n";
volatile unsigned char UART_RX_Count;


volatile unsigned char CurrentChannel, CurrentDataRate, CurrentUartBaud, CurrentRxPowerLevel,CurrentTxPowerLevel;
//volatile signed char CurrentPower;
volatile signed char CurrentPowerLevel; //V1.5
volatile unsigned long CurrentFrequency;
volatile unsigned long CurrentCpuFrequency;  //Added V1.5
volatile unsigned int CurrentGroupID;
volatile unsigned char CurrentAESHasEnabled;
volatile unsigned char CurrentLPMLevel; //Added in V1.5
volatile unsigned char CurrentCarrierMode;  //Added in V1.5

volatile unsigned char CurrentCPULPMLevel; //Added in V1.5

volatile unsigned char RecoveryPin; //V1.5

volatile unsigned char CurrentHandshaking; //V1.5

volatile unsigned int CurrentCarrierDetectRSSI; //V1.5.5

volatile unsigned char CDPolarity; //V1.5.5

volatile signed char CDRSSI_Hex; //V1.5.5


//volatile unsigned int LED_Timer = 1; //V1.4
volatile unsigned long LED_Timer = 1;  //V1.5
//volatile float LED_Timer_Timeout = 0; //V1.5

volatile unsigned long LED_Timer_Timeout_1s = 0; //V1.5.5
volatile unsigned long LED_Timer_Timeout_1ms = 0; //V1.5.5

volatile unsigned char Bool_INLPM_LOWCPU = 0;//V1.5.5 THis is set when LPM MODES are set and PIn22 is low or CPU is <1000000

volatile unsigned long CommandTimeout_Length = 0;   //V1.5.5 THis is used not to deliver data when Command is sent.

volatile unsigned long CommandTimeout_200ms = 0;   //V1.5.5 THis is used not to deliver data when Command is in process or before ACK is sent.

volatile unsigned char WaitingForACK = 0;

//volatile unsigned char CommandTimeout = 0;
volatile unsigned long CommandTimeout = 0; //V1.5.5

const long BaudValues[] = {1200,2400,4800,9600,19200,38400,31250,76800,115200} ;

volatile signed char PowerValue[] = {0,1,2,3,4,5,6,7,8,9}; //These are values in dbm V1.5


void EasyRadio_Receive_routine();  //V1.5
void EasyRadio_Transmit_routine(); //V1.5

void SetLPM(volatile unsigned char level); //V1.5 V4.1

int main(void)
{
    eRIC_WDT_Stop();

    Pin3_FunctionUartATxOUT();          //Map pin3 as Uart TX Moved to here in V1.5.5
    Pin4_FunctionUartARxD();            //Map Pin4 as Uart Rx Moved to here in V1.5.5

	eRIC_GlobalInterruptDisable();      //  __bic_SR_register(GIE);

	Initialise_Globalvariables();      //Initialises all global variable in this function

	Pin16_SetAsInput();
	Pin16_PullDownEnable();
	Pin11_SetAsInput(); //V1.5
	Pin11_PullDownEnable();//V1.5
	if(Pin11_Read())        //V1.5
	    RecoveryPin = 1;              //This will set CPU to 4.5 and baud rate to 19200 just for recovery.


	//This is to enable Pin 22 as interrupt to exit LPM modes
	Pin22_PullDownEnable();  //V1.5
	Pin22_InterruptLow2High();  //V1.5
	Pin22_ClearInterruptFlag();  //V1.5

	eROS_Initialise(CurrentFrequency);  //Initialises eROS and sets radio parameter to default at CurrentFrequency

	Pin2_CTS();
	//Pin5_RTS();                       //Enabled in Radio stored settings
	Pin1_SetAsOutput();                 //Pin1 set as output for Carrier detect


    Radio_Stored_settings();            //This function will restore all previous radio settings before power off from Eeprom

	if(SW1Read())                       //Runs demo mode if Switch1 on dev board is high or pressed
	{
		while(SW1Read());               //Waits here until Switch1 on dev board goes low or released

		Pin19_SetAsOutput();            //Pin19 set as output
		Pin18_SetAsOutput();            //Pin18 is set as output
		Pin18_SetHigh();                //Pin18 is high to indicate it entered demo mode
		Pin19_SetHigh();                //Pin19 is high to indicate it entered demo mode
		eRIC_Delay(500);//__delay_cycles(2250000);        //delay 500 ms    (2250000/4.5(CPU Speed))
		Pin18_SetLow();                 //Pin18 set low
		Pin19_SetLow();                 //Pin19 set low
		eRIC_Delay(500);//__delay_cycles(2250000);        //delay 500 ms
		Pin18_SetHigh();                //Pin18 is high to indicate it entered demo mode
		Pin19_SetHigh();                //Pin19 is high to indicate it entered demo mode
		eRIC_Delay(500);//__delay_cycles(2250000);        //delay 500 ms    (2250000/4.5(CPU Speed))
		Pin18_SetLow();                 //Pin18 set low
		Pin19_SetLow();                 //Pin19 set low

		Demo_Mode();
	}

	if(CurrentRxPowerLevel!=8)//  V1.5 after adding disable Rx  function
	{
	    eRIC_Rx_Enable();       //Turns on Radio receiver
	}
	else
	{
	    eRIC_Rx_Disable();
	   // eRIC_RadioSleep();
	}
	Pin16_SetAsOutput();  //This is for blinking led every 1 second
	Pin16_SetHigh();


	SetupTimer();

	if(CurrentCarrierMode>0 && CurrentCarrierMode<4) //V1.5
	{
	    SetCarrierModeOnStartup(CurrentCarrierMode);
	}

	eRIC_GlobalInterruptEnable();  //Enables all Global interrupts

	eRIC_RfDataReceivedInterruptEnable(); //V1.5.4 When this interrupt is enabled, eRIC_RfDataReceivedInterrupt(); function is interrupted whenever there is some data received over air
	easyRadio();
}


void Initialise_Globalvariables()
{

	Uart_buff_Timeout = 0;
	Uart_Timeout = 0;
	UART_RX_Count = 0;
	CurrentChannel = 0;
	CurrentDataRate = 5;
	CurrentUartBaud = 4;
	CurrentRxPowerLevel = 0;


	CurrentTxPowerLevel = 0;
	CurrentPowerLevel = 9;
	CurrentFrequency = ModuleFreq;
	CurrentCpuFrequency = 4500000; //Default for eric app
	//CurrentCpuFrequency = 1048576; //Default in cc430 and eros
	CurrentGroupID = 0;

	CurrentAESHasEnabled = 0;

	CurrentLPMLevel = 0; //Added in V1.5
	CurrentCPULPMLevel = 0; //Added in V1.5
	RecoveryPin = 0; //V1.5
	CurrentCarrierMode = 0;  //Added in V1.5
	CurrentHandshaking = 0; //V1.5

	CurrentCarrierDetectRSSI = 0x00FF;  //V1.5.5 int

    CDPolarity = 0;               //V1.5.5
    CDRSSI_Hex = 0xFF;               //V1.5.5

	CommandTimeout = 0;//V1.5.4

	volatile signed char i =0;  //V1.5


#ifdef eRIC9
#ifdef limitOnPower_FCC  //V1.5
        if(CurrentFrequency>=869750000 && CurrentFrequency<=870000000)
        {
            while(i<10)
            {
                PowerValue[i] = i-2; //Default settings are -2 - 7 values
                i++;
            }
        }
        else if(CurrentFrequency>=902000000 && CurrentFrequency<=928000000)
        {
            while(i<10)
            {
                PowerValue[i] = i-12; //Default settings are -12 - -3 values
                i++;
            }
        }
#else
        while(i<10)
        {
            PowerValue[i] = i; //Default settings are 0-9 values
            i++;
        }
#endif
#else
while(i<10)
{
    PowerValue[i] = i; //Default settings are 0-9 values
    i++;
}
#endif
}



void Demo_Mode()
{
	SW1Enable();
	SW2Enable();
	SW3Enable();
	SW4Enable();
	eRIC_Rx_Disable();//V1.5
	eRIC_RadioSleep();//V1.5
	eRIC_GlobalInterruptEnable(); //V1.5.1
//	eRIC_Power = 12;                    //Demo mode uses max power, so that range tests can also be performed
//	eRIC_RadioUpdate();
	while(1)
	{
		if(SW1Read())                  //Return to EasyRadio companion Mode
		{
			while(SW1Read());          //Wait while Switch1 is released
			Pin17_SetAsOutput();       //Pin17 is set as output
			Pin18_SetAsOutput();       //Pin18 is set as output
			Pin17_SetHigh();           //Pin17 is set high to indicate easyradio mode is running
			Pin18_SetHigh();           //Pin18 is set high to indicate easyradio mode is running
			eRIC_Delay(500);//__delay_cycles(2250000);   //delay 500 ms
			Pin17_SetLow();            //Pin17 is set low
			Pin18_SetLow();            //Pin18 is set low
			eRIC_Delay(500);//__delay_cycles(2250000);   //delay 500 ms
			Pin17_SetHigh();           //Pin17 is set high again
			Pin18_SetHigh();           //Pin18 is set high again
			eRIC_Delay(500);//__delay_cycles(2250000);   //delay 500 ms
			Pin17_SetLow();            //Pin17 is set low
			Pin18_SetLow();            //Pin18 is set low
			return;
		} // end of if SW1

		// SW2 Demo sends each switch as a simple value over air. Sends a change of any switch.
		if(SW2Read())
		{
			volatile unsigned char pins = 0, pinslast = 0;
			while(SW2Read());
			while(1)
			{
				while(eRIC_RadioTx_BuffCount);// wait for last TX to finish
				pins = 0;
				if(SW1Read())
					pins |= 1;
				if(SW2Read())
					pins |= 2;
				if(SW3Read())
					pins |= 4;
				if(SW4Read())
					pins |= 8;
				if(pins != pinslast)
				{
					pinslast = pins;
					eRIC_RadioTx_Buffer[eRIC_RadioTx_BuffCount++] = pins;
					eRIC_RfSenddata(); // Transmits the data
				}
			}
		}
		// SW3 Listens and sets LED from message from a transmitter on mode 2 or 4
		if(SW3Read())
		{
			while(SW3Read());
			LED1Enable();
			LED2Enable();
			LED3Enable();
			LED4Enable();
			eRIC_Rx_Enable();
			volatile unsigned char pins;
			while(1)
			{
				if(eRIC_Rxdata_available)
				{
					if(eRIC_Rxdata_available == 1)
					{
						pins = eRIC_ReadRfByte();
						if(pins & 1)
							LED1_Set();
						else
							LED1_Clear();
						if(pins & 2)
							LED2_Set();
						else
							LED2_Clear();
						if(pins & 4)
							LED3_Set();
						else
							LED3_Clear();
						if(pins & 8)
							LED4_Set();
						else
							LED4_Clear();
					}
					else
					{
						while(eRIC_Rxdata_available)// = 0; // Data has been dealt with or rejected
						{
							pins = eRIC_ReadRfByte();
						}
					}
				}
			}
		}
		// Counter, that increments and transmits a single byte. Lower nibble is decoded to the light on a receiver on mode 3
		if(SW4Read())
		{
			while(SW4Read());
			LED1Enable();
			LED2Enable();
			LED3Enable();
			LED4Enable();
			while(1)
			{
				static unsigned char counter = 0;
				eRIC_Delay(125);//__delay_cycles(562500);  //125ms at 4.5M clock
				counter++;
				if(counter & 1)
					LED1_Set();
				else
					LED1_Clear();
				if(counter & 2)
					LED2_Set();
				else
					LED2_Clear();
				if(counter & 4)
					LED3_Set();
				else
					LED3_Clear();
				if(counter & 8)
					LED4_Set();
				else
					LED4_Clear();
				eRIC_RadioTx_Buffer[eRIC_RadioTx_BuffCount++] = counter;
				eRIC_RfSenddata(); // Transmits the data
			}
		}
	}// End While
}

/*This function reads all data from eeprom which was stored before power down and updates radio settings to that*/
void Radio_Stored_settings()
{
    char interruptBool = eRIC_GlobalInterruptIsEnabled();//Get GIE interrupt status
    eRIC_GlobalInterruptDisable(); //Disable GIE
	volatile char i =0;
	if((eRIC_Eeprom_Read(0) < 2) || (eRIC_Eeprom_Read(0)==0xFF))//if there are no stored radio settings, writes Eeprom with default radio settings
	{
		Initialise_Globalvariables();

		eRIC_Eeprom_Write(1,CurrentDataRate);                   //Writes default baudrate 5(38400)  into Eeprom address 1
		eRIC_Eeprom_Write(2,CurrentChannel);                    //Writes default channel which is 0 into Eeprom address 2
		eRIC_Eeprom_Write(3,CurrentPowerLevel);                 //Writes default power which is 9 into Eeprom address 3
		eRIC_Eeprom_Write(4,CurrentUartBaud);                   //Writes default Uartbaudrate which is 4(19200) into Eeprom address 4
		eRIC_Eeprom_Write(5,CurrentRxPowerLevel);               //Writes default Rxpowerlevel which is 0 into Eeprom address 5
		eRIC_Eeprom_Write(6,CurrentTxPowerLevel);               //Writes default Txpowerlevel which is 0 into Eeprom address 6
		eRIC_Eeprom_Write(11,CurrentGroupID>>8);                //Writes default GroupID 00 into Eeprom address 11
		eRIC_Eeprom_Write(12,CurrentGroupID);                   //Writes default GroupID 00 into Eeprom address 12

 	     //Frequency reading
		eRIC_Eeprom_Write(7,(CurrentFrequency>>24));            //Eeprom address 7 for Frequency and writes default
		eRIC_Eeprom_Write(8,(CurrentFrequency>>16));            //Eeprom address 8 for Frequency and writes default
		eRIC_Eeprom_Write(9,(CurrentFrequency>>8));             //Eeprom address 9 for Frequency and writes default
		eRIC_Eeprom_Write(10,CurrentFrequency);                 //Eeprom address 10 for Frequency and writes default

		//AES
		eRIC_Eeprom_Write(13,CurrentAESHasEnabled);             //Eeprom address 13 has AES ENABLED Or not info, by default it is 0(Not enabled)


		//CPUFrequency reading
		eRIC_Eeprom_Write(14,(CurrentCpuFrequency>>24));        //Eeprom address 14 for Frequency and writes default
		eRIC_Eeprom_Write(15,(CurrentCpuFrequency>>16));        //Eeprom address 15 for Frequency and writes default
		eRIC_Eeprom_Write(16,(CurrentCpuFrequency>>8));         //Eeprom address 16 for Frequency and writes default
		eRIC_Eeprom_Write(17,CurrentCpuFrequency);              //Eeprom address 17 for Frequency and writes default

		eRIC_Eeprom_Write(18,CurrentLPMLevel);                  //Eeprom address 18 for Low power level V1.5

		eRIC_Eeprom_Write(19,CurrentCarrierMode);               //Eeprom address 19 for Carrier level V1.5


		i =0 ;
		while(i<10)
		{
		    eRIC_Eeprom_Write(20+i,PowerValue[i]);          //Eeprom address 20 - 28 for power exact data in dbm
		    i++;
		}

		eRIC_Eeprom_Write(30,CurrentHandshaking);              //Eeprom address 30 for handshaking 1.5

		eRIC_Eeprom_Write(31,CurrentCPULPMLevel);              //Eeprom address 31 for CPU Low power level V1.5

		eRIC_Eeprom_Write(32,CurrentCarrierDetectRSSI>>8);        //Eeprom address 32 for current carrier detect V1.5.5
		eRIC_Eeprom_Write(33,CurrentCarrierDetectRSSI);        //Eeprom address 32 for current carrier detect V1.5.5
		eRIC_Eeprom_Write(0,2);                                 //Change eeprom(0) to 2, so that it will not load default settings again
	}

	{

	    //Gets CPUCLOCKFrequency value from Eeprom location
	    CurrentCpuFrequency = eRIC_Eeprom_Read(14);            //Reads Eeprom location 14 and assigns to current CPU frequency high byte
	    CurrentCpuFrequency<<=8;
	    CurrentCpuFrequency |= eRIC_Eeprom_Read(15);           //Reads Eeprom location 15 and assigns to current CPU frequency next high byte
	    CurrentCpuFrequency<<=8;
	    CurrentCpuFrequency |= eRIC_Eeprom_Read(16);           //Reads Eeprom location 16 and assigns to current CPU frequency next high byte
	    CurrentCpuFrequency<<=8;
	    CurrentCpuFrequency |= eRIC_Eeprom_Read(17);          //Reads Eeprom location 17 and assigns to current CPU frequency low byte
	    if(RecoveryPin)
	        CurrentCpuFrequency = 4500000;
        if(eRIC_CpuFrequency!=CurrentCpuFrequency)
            eRIC_SetCpuFrequency(CurrentCpuFrequency);


		CurrentDataRate = eRIC_Eeprom_Read(1);//Eeprom read location 1 and set to current datarate
		switch(CurrentDataRate)//There is no 10000 and 175000 baud just to fit 1200-500k in 0-9
		{
		case 0:
			eRIC_RfBaudRate = 1200;
			break;
		case 1:
			eRIC_RfBaudRate = 2400;
			break;
		case 2:
			eRIC_RfBaudRate = 4800;
			break;
		case 3:
			eRIC_RfBaudRate = 9600;
			break;
		case 4:
			eRIC_RfBaudRate = 19200;
			break;
		case 5:
			eRIC_RfBaudRate = 38400;
			break;
		case 6:
			eRIC_RfBaudRate = 76800;
			break;
		case 7:
			eRIC_RfBaudRate = 100000;
			break;
		case 8:
			eRIC_RfBaudRate = 250000;
			break;
		case 9:
			eRIC_RfBaudRate = 500000;
			break;
		default:
			eRIC_RfBaudRate = 38400;
			break;
		}
		CurrentChannel  = eRIC_Eeprom_Read(2);              //Reads Eeprom location 2 and assigns to current channel
		eRIC_Channel = CurrentChannel;
		CurrentRxPowerLevel = eRIC_Eeprom_Read(5);          //Reads Eeprom location 5 and assigns to current Rxpowerlevel
		eRIC_RxPowerLevel = CurrentRxPowerLevel;
		CurrentTxPowerLevel = eRIC_Eeprom_Read(6);          //Reads Eeprom location 6 and assigns to current Txpowerlevel
	    eRIC_TxPowerLevel = CurrentTxPowerLevel;

		//Gets Frequency value from Eeprom location
		CurrentFrequency = eRIC_Eeprom_Read(7);            //Reads Eeprom location 7 and assigns to current frequency high byte
		CurrentFrequency<<=8;
		CurrentFrequency |= eRIC_Eeprom_Read(8);           //Reads Eeprom location 8 and assigns to current frequency next high byte
		CurrentFrequency<<=8;
		CurrentFrequency |= eRIC_Eeprom_Read(9);           //Reads Eeprom location 9 and assigns to current frequency next high byte
		CurrentFrequency<<=8;
		CurrentFrequency |= eRIC_Eeprom_Read(10);          //Reads Eeprom location 10 and assigns to current frequency low byte
		eRIC_Frequency = CurrentFrequency;



		CurrentPowerLevel = eRIC_Eeprom_Read(3);                //Reads Eeprom location 3 and assigns to current power
		i =0;
		while(i<10)
		{
		    PowerValue[i] = eRIC_Eeprom_Read(20+i);
		    i++;
		}

		Power(); //V1.5

		CurrentGroupID = eRIC_Eeprom_Read(11);            //Reads Eeprom location 11 and assigns to current groupid high byte
		CurrentGroupID <<=8;
		CurrentGroupID |= eRIC_Eeprom_Read(12);          //Reads Eeprom location 12 and assigns to current groupid low byte
		eRIC_GroupID = CurrentGroupID;
		if(eRIC_GroupID)
			eRIC_GroupIDEnable(eRIC_GroupID);
		eRIC_RadioUpdate();

		if(RecoveryPin)
		    CurrentUartBaud = 4;
		else
		    CurrentUartBaud = eRIC_Eeprom_Read(4);          //Reads Eeprom location 4 and assigns to current datarate
		//eRIC_UARTA_SetBaud(BaudValues[CurrentUartBaud]);    ///V1.4
		eRIC_UARTAInitialise(BaudValues[CurrentUartBaud]); //V1.5
		eRIC_UartARxInteruptEnable();

		CurrentAESHasEnabled = eRIC_Eeprom_Read(13);      //Reads Eeprom location 13 and assigns to current AESHasEnabled, by default it is 0

		if(CurrentAESHasEnabled)
		{
		    if(CurrentAESHasEnabled==3) //CurrentAESHasEnabled is 1 or 2 for encryption and 3 for decryption
		        eRIC_AES_Key[0] = 1; //Decryption mode
		    else
		        eRIC_AES_Key[0] = 0; //Encryption mode
		    eRIC_AES_SetKey();
		}

		RecoveryPin = 0;
		CurrentCarrierMode = eRIC_Eeprom_Read(19); //V1.5 read Eeprom location 19 and assigns carrier level

		CurrentHandshaking = eRIC_Eeprom_Read(30);//V1.5 read Eeprom location 30 for handshaking
		if(CurrentHandshaking)
		{
		    Pin2_CTS();
		    Pin5_RTS();
		}

		CDPolarity = CurrentCarrierDetectRSSI = eRIC_Eeprom_Read(32); //V1.5.5
		CurrentCarrierDetectRSSI <<=8;
		CurrentCarrierDetectRSSI += eRIC_Eeprom_Read(33);  //V1.5.5
		CDRSSI_Hex = CurrentCarrierDetectRSSI&0xFF;

	//	CurrentLPMLevel = eRIC_Eeprom_Read(18); //V1.5 read Eeprom location 18 and assigns LPM level

	//	CurrentCPULPMLevel = eRIC_Eeprom_Read(31); //V1.5 read Eeprom location 31 and assigns CPULPM level

        SetLedtimerTimeOut();
		SetUARTTimeout();

		/*if(!Pin22_Read())  //Added in V1.5.1//This needs to be at end ,as Uarttimeouts are set based in CurreneLPMlevels
		{
		    CurrentLPMLevel = eRIC_Eeprom_Read(18);
		    CurrentCPULPMLevel = eRIC_Eeprom_Read(31);
		}*/

	}
	if(interruptBool)
	    eRIC_GlobalInterruptEnable();
}

void    SetUARTTimeout()
{
	volatile long UARTbitRate = BaudValues[CurrentUartBaud]/20; //Modified to long from float in V1.5.5 //V1.5  removed bitRates and replaced by BaudValues as BaudValues already defined


	if(Bool_INLPM_LOWCPU)//V1.5.5 THis is set when LPM MODES are set and PIn22 is low or CPU is <1000000//(CurrentLPMLevel || eRIC_CpuFrequency<1000000 ||CurrentCPULPMLevel)
	{
	    Uart_Timeout = (eRIC_CpuFrequency/32)/UARTbitRate;     //No division by 100, as we dont rely on timer which is set at 100cycles. But Uart interrupt loop it takes around 28 cycles for one while(Uartbuftimeout--),so nearest easy divider 32
	}
	else
	{
	    Uart_Timeout = (eRIC_CpuFrequency/100)/UARTbitRate;//it is divided by 100. as timer is set for every 100 cycles.
	}

	CommandTimeout_Length = (eRIC_CpuFrequency/100)/UARTbitRate;//V1.5.5 //it is divided by 100. as timer is set for every 100 cycles.
	CommandTimeout_200ms = eRIC_CpuFrequency/500; // (200000*eRIC_CpuFrequency)/1000000/100 //V1.5.5 //it is divided by 100. as timer is set for every 100 cycles.
}




// All the functions to work with Companion software are here. Simple UART data is also transmitted
void easyRadio()
{
    volatile unsigned char LastPin14 = 5;

    #ifdef SwapFrequency_Pin14
	{
	    Pin14_SetAsInput();        //Pin14 is set as input for swapping frequency
	    Pin14_PullUpEnable();
	}
	#endif

	while(1)
	{
	    #ifdef SwapFrequency_Pin14
	    // check switch position
	    if(LastPin14 != Pin14_Read())
	    {
	        LastPin14 = Pin14_Read();
	        if(LastPin14)
	        {
	            //Frequency value from Eeprom location
	            CurrentFrequency = eRIC_Eeprom_Read(7);//Reads Eeprom location 7 and assigns to current frequency high byte
	            CurrentFrequency<<=8;
	            CurrentFrequency |= eRIC_Eeprom_Read(8);//Reads Eeprom location 8 and assigns to current frequency next high byte
	            CurrentFrequency<<=8;
	            CurrentFrequency |= eRIC_Eeprom_Read(9);//Reads Eeprom location 9 and assigns to current frequency next high byte
	            CurrentFrequency<<=8;
	            CurrentFrequency |= eRIC_Eeprom_Read(10);//Reads Eeprom location 10 and assigns to current frequency low byte
	            CurrentPowerLevel = eRIC_Eeprom_Read(3);
	        }
	        else
	        {
	            if(CurrentFrequency<902000000)  //V1.5
	                CurrentFrequency = 915000000;
	        }
	        eRIC_Frequency = CurrentFrequency;

	        Power(); //V1.5



	        eRIC_RadioUpdate();
	        //eRIC_Rx_Enable();            //Turn radio receiver on
	        if(CurrentRxPowerLevel!=8)//  V1.5 after adding disable Rx  function
	            eRIC_Rx_Enable();       //Turns on Radio receiver
	        if(CurrentCarrierMode>0 && CurrentCarrierMode<4) //V1.5
	        {
	            SetCarrierModeOnStartup(CurrentCarrierMode);
	        }
	    }
	    #endif
	    if(!Pin22_Read())  //Added in V1.5.1
	    {
	        CurrentLPMLevel = eRIC_Eeprom_Read(18);
	        CurrentCPULPMLevel = eRIC_Eeprom_Read(31);
	        if(CurrentLPMLevel||CurrentCPULPMLevel) //V1.5.1
	        {
	            if(!CurrentCPULPMLevel)
	                SetLPM(CurrentLPMLevel);  //Set low power level and enters one of the LPM modes
	            else
	                SetLPM(CurrentCPULPMLevel); //Set low power level and enters one of the LPM modes
	        }
	    }

	    if(eRIC_CpuFrequency>999999)  //Do not do this(Check RSSI) at low clock speeds.
	    {
	        /*Carrier detect code below*/
	       if(CurrentRxPowerLevel==0)  //This 'if' condition is added in V1.4 to prevent ER_CMD#D7 commands stop working
	        {
	           if(CDRSSI_Hex!=0xFF)   //Disable CD if ER_CMD#a1xFF
	           {
	           if(CDRSSI_Hex<eRIC_GetLiveRSSI())//eRIC_RadioRegRead(RSSI) //Added V1.5.5, if live RSSI is greater than set RSSI through command
	           {
	               if(CDPolarity)           //IF in ER_CMD#a1xyy, x=1 then it is low when Carrier detect
	                   Pin1_SetLow();
	               else
	                   Pin1_SetHigh();
	           }
	           else
	           {
	               if(CDPolarity)         //IF in ER_CMD#a1xyy, x=1 then it is High when No Carrier detect
	                   Pin1_SetHigh();
	               else
	                   Pin1_SetLow();
	           }
	           }//End of if(CDRSSI_Hex!=0xFF)
	        }//end of if(CurrentRxPowerLevel==0)
	        /*End of carrier detect code*/
	        EasyRadio_Transmit_routine();
	    }
	} // While(1)
}


/*This function is added in V1.5. Before this code was in easyradio() and now its been added in EasyRadio_Receive_routine()
 * function to look clean. THis function checks if the data needs AES to be performing before sending to uart
 */
//void EasyRadio_Receive_routine()  //V1.5
void eRIC_RfDataReceivedInterrupt()
{
    if(CommandTimeout)  //V1.5.4 , Data is not delivered until command is executed..
        return;
    volatile unsigned char Bytesinpacket = 0;
    if(eRIC_Rxdata_available>0)
    {
        if(Pin5_Read() && CurrentHandshaking)//RTS handshake V1.5
        {
            CommandTimeout = CommandTimeout_Length;//Added V1.5.5 check every 20 bits delay even if Handshaking is on.
            return;  //Do not deliver data when Pin5 is high and handshaking enabled
        }
        if(CurrentAESHasEnabled == 1 || CurrentAESHasEnabled == 3)//IF Both AES is enabled or only AES Decryption mode is selected
        {
            if(eRIC_AES_Key[0]!=1)//(CurrentAES_ModeandKey[0]!=1) //If AES is not in decryption mode
            {
                eRIC_AES_Key[0]=1;
                eRIC_AES_SetKey();  //Set in decryption mode
            }
            volatile unsigned char i = 0,j = 0;
            volatile char counttimeout =0;
            Bytesinpacket = eRIC_ReadRfByte();//Reads the byte count of the overair packet which is first byte of packet
            for(i=0;i<Bytesinpacket;i+=16)
            {
                j=0;
                while(j<16)
                {
                    if(eRIC_Rxdata_available)
                    {
                        counttimeout =0;
                        eRIC_AES_Data[j] = eRIC_ReadRfByte();   //Reads each byte upto 16 bytes
                        j++;
                    }
                    else
                    {
                        counttimeout++;    //If there is no byte then it waits for 10 ms
                        eRIC_Delay(10);
                    }
                    if(counttimeout==50) //If there is no data for 500ms break this complete
                    {
                        while(eRIC_Rxdata_available)
                            eRIC_ReadRfByte();   //FLush all bytes
                        j =16;
                    }
                }
                if(counttimeout==50) //If there is no data for 500ms break this complete
                {
                    break;
                }
                eRIC_AES_Run();  //Decrypt packet of 16 bytes
                j =0;
                while(((i+j)<Bytesinpacket) && (j<16))
                {
                    eRIC_UartASendByte(eRIC_AES_Data[j]);        //send decrypted packet to Uart
                    while(eRIC_UartATxBufferIsBusy());
                    j++;
                }
            }
        }
        else
        {
            if(Bool_INLPM_LOWCPU)//V1.5.5 THis is set when LPM MODES are set and PIn22 is low or CPU is <1000000
            {
                eRIC_WaitForStableClock();  //V1.5.5
                while(eRIC_Rxdata_available)
                {
                    eRIC_UartASendByte(eRIC_ReadRfByte());
                    while(eRIC_UartATxBufferIsBusy());
                }
                while(eRIC_UartABusy());  //Added V1.5.4
                return;
            }
            if(! eRIC_UartATxIsEnabled()) // Deliver the data
            {
                eRIC_UartATxInteruptEnable();
            }
        }
    }
}

/*This function is added in V1.5. Before this code was in easyradio() and now its been added in EasyRadio_Transmit_routine()
 * function to look clean. THis function checks if the data is command or normal data and does it need AES to be performing
 *  and sends over air accordingly.
 */

void EasyRadio_Transmit_routine() //V1.5
{
    if(UART_RX_Count)
    {
        if(Uart_buff_Timeout>Uart_Timeout)
        {
            if(!ER_Command(UART_RX_Buffer,UART_RX_Count))
            {
                eRIC_RfDataReceivedInterruptEnable(); //V1.5.5 its been enabled

                if(CurrentCarrierMode) //V1.5 if any carrier is already on do not send data over as it hangs in eros.
                {
                    UART_RX_Count = 0; // Data has been dealt with or rejected
                    return;
                }
                if(CurrentAESHasEnabled == 1 || CurrentAESHasEnabled == 2)//IF Both AES is enabled or only AES encryption mode is selected
                {
                    volatile unsigned char i,j;
                    if(eRIC_AES_Key[0]!=0)//(CurrentAES_ModeandKey[0]!=0) //If AES is not in encryption mode
                    {
                        eRIC_AES_Key[0]=0;
                        eRIC_AES_SetKey();  //Set in decryption mode
                    }
                    eRIC_RadioTx_Buffer[0] = 0;
                    while(UART_RX_Count)
                    {
                        if(UART_RX_Count>240)//Generally packet should be ok upto250 bytes but since AES using 16bytes in one go after 240 bytes the next byte will have 16 bytes which makes it 256bytes,so
                            //In order to avoid overloading, anything more than 240 bytes it send first 16 bytes and then it carries on with rest of it
                        {
                            eRIC_RadioTx_Buffer[0] = 240;  //Send first byte as 240 which is byte count of packet
                            eRIC_Stringcopy(&eRIC_RadioTx_Buffer[1],UART_RX_Buffer,eRIC_RadioTx_Buffer[0]);  //Copy all first 240 bytes from Uart to buffer to send overair
                        }
                        else
                        {
                            eRIC_Stringcopy(&eRIC_RadioTx_Buffer[1],&UART_RX_Buffer[eRIC_RadioTx_Buffer[0]],UART_RX_Count); //Copy all rest of the bytes in to overair transmit buffer
                            eRIC_RadioTx_Buffer[0] = UART_RX_Count;       //Send first byte as byte count of rest of the packet
                        }
                        for(i=0;i<eRIC_RadioTx_Buffer[0];)
                        {
                            for(j=0;j<16;j++)
                                eRIC_AES_Data[j] = eRIC_RadioTx_Buffer[j+i+1]; //copy into AES_DATA to encrypt
                            eRIC_AES_Run();   //encrypt packet in chunks of 16 bytes and encrypted is saved in same buffer
                            for(j=0;j<16;j++)
                                eRIC_RadioTx_Buffer[j+i+1] = eRIC_AES_Data[j]; //coopy back encrypted data in Tx buffer to transmit
                            i+=16;
                        }
                        eRIC_RadioTx_BuffCount = i+1; //Over air byte count will be 17 which is 16 bytes of encrypted data plus byte count of packt which will first byte
                        eRIC_RfSenddata();  //Send overair
                        UART_RX_Count-=eRIC_RadioTx_Buffer[0];  //Reduce Uart byte count to leftover as few bytes have been sent already
                        }
                }
                else//If AES is not enabled
                {
                    // Copy UART buffer to tx buffer
                    eRIC_Stringcopy(eRIC_RadioTx_Buffer,UART_RX_Buffer,UART_RX_Count);
                    eRIC_RadioTx_BuffCount = UART_RX_Count;
                    eRIC_RfSenddata(); // Transmits the data
                }
            }
            else   //V1.5.5
            {//if the bytes recieved through UART is command
                if(!(CurrentLPMLevel||CurrentCPULPMLevel))//Dont timeout at LPM as timer doesnt work
                {
                    if(WaitingForACK)
                        CommandTimeout = CommandTimeout_200ms;  //V1.5.5 Timeout of 200 ms wait before delivering data for commands which needs ACK
                    else
                        CommandTimeout = CommandTimeout_Length; //V1.5.5
                }
                eRIC_RfDataReceivedInterruptEnable(); //V1.5.5 its been enabled

            }
            UART_RX_Count = 0; // Data has been dealt with or rejected
       }
    }
}

/***********************
 * THis function is used to WAKE up fro LPM modes. Its based on PIn22
 */

#pragma vector= PORT2_VECTOR       //Added V1.5
__interrupt void PORT2_ISR(void)
{
    volatile unsigned int function =  P2IV;     // Read Interrupt Vector triggered
    char temp = 50;
    while(temp--)
    {
    if(!(Pin22_Read())) //return if pin22 is low
        return;
    }
    switch(function)
    {
    case 0x02:
        break;
    case 0x04:
            break;
    case 0x06:
            break;
    case 0x08:
            break;
    case 0x0A:
            break;
    case 0x0C: //If interrupted on PORT 2.5
    {
        if(CurrentLPMLevel == 1 || CurrentCPULPMLevel == 1)
        {
            eRIC_LPM_ExitLevel0();
        }
        else if(CurrentLPMLevel == 2 || CurrentCPULPMLevel == 2)
        {
            eRIC_LPM_ExitLevel1();
        }
        else if(CurrentLPMLevel == 3 || CurrentCPULPMLevel == 3)
        {
            eRIC_LPM_ExitLevel2();
        }
        else
        {
        }
    }
    break;
    default:
        break;
    }
}


int ER_Command(char * buff, unsigned char count)
{
//    static unsigned char WaitingForACK = 0;

	//static char CMD[9];
	//static char CMD[20]; // changed from 9 - 20 for AES command, A1yx...16x  A1 is AES command and next y is on or 0 is off and y is 1 for both encryption and decryption and y is mode 2 for only encryption 3 for only decryption and 16xs are key
	static char CMD[36]; // V1.5changed from 20 to 36 for AES command, A1yx...32x  A1 is AES command and next y is on or 0 is off and y is 1 for both encryption and decryption and y is mode 2 for only encryption 3 for only decryption and 32xs are key

	volatile unsigned char CurrentVal = 0; // This is the return value for ? commands
	if(WaitingForACK)
	{
	    volatile unsigned long Tempfrequency = 0;
	    WaitingForACK = 0;
	    if(0 == eRIC_Stringcompare(buff, "ACK",3))
	    {
	        switch(CMD[0])
	        {
	        case 'a':
	        {
	            if(CMD[1]=='1')//Carrier Detect RSSI V1.5.5
	            {
	                if(CMD[2] == '?')
	                {
	                    eRIC_UartASendByte(((CurrentCarrierDetectRSSI>>8)&0x0F) +'0');
	                    while(eRIC_UartATxBufferIsBusy());

	                    if(((CurrentCarrierDetectRSSI>>4)&0x0F)<10)
	                        eRIC_UartASendByte(((CurrentCarrierDetectRSSI>>4)&0x0F)+'0');
	                    else
	                        eRIC_UartASendByte(((CurrentCarrierDetectRSSI>>4)&0x0F)+'7');
	                    while(eRIC_UartATxBufferIsBusy());

	                    if((CurrentCarrierDetectRSSI&0x0F)<10)
	                        eRIC_UartASendByte((CurrentCarrierDetectRSSI&0x0F)+'0');
	                    else
	                        eRIC_UartASendByte((CurrentCarrierDetectRSSI&0x0F)+'7');
	                    while(eRIC_UartATxBufferIsBusy());
	                }
	                else
	                {
	                    CurrentCarrierDetectRSSI = eRIC_Ascii2Hex(CMD[2]);
	                    CurrentCarrierDetectRSSI<<=4;
	                    CurrentCarrierDetectRSSI &=~0x0F;
	                    CurrentCarrierDetectRSSI += eRIC_Ascii2Hex(CMD[3]);
	                    CurrentCarrierDetectRSSI<<=4;
	                    CurrentCarrierDetectRSSI &=~0x0F;
	                    CurrentCarrierDetectRSSI += eRIC_Ascii2Hex(CMD[4]);

	                    eRIC_Eeprom_Write(32,CurrentCarrierDetectRSSI>>8);
	                    eRIC_Eeprom_Write(33,CurrentCarrierDetectRSSI&0xFF);

	                    CDPolarity = CurrentCarrierDetectRSSI>>8;
	                    CDRSSI_Hex = CurrentCarrierDetectRSSI&0xFF;
	                }
	            }
	            else
	            {
	             //Do Nothing
	            }
	        }
	        break;
	        case 'A':
	        {
	            if(CMD[1]=='1')//AES encryption/Decryption
	            {
	                //AES command is explianed below:
	                //ER_CMD#A1yx...16x where A1 is AES command and when y is 0 Aes is off and y is 1 AES is on with encryption and decryption and when y is 2 it on only foe encryption and y is 3 is on only for decryption
	                volatile char i =0;
	                if(CMD[2] >='0' && CMD[2] <'4') //Encryption/Decryption ON or OFF
	                {
	                    CurrentAESHasEnabled = eRIC_Ascii2Hex(CMD[2]);
	                    eRIC_Eeprom_Write(13,CurrentAESHasEnabled);
	                    if(CurrentAESHasEnabled)
	                    {
	                        if(CurrentAESHasEnabled==3) //decryption
	                            eRIC_AES_Key[0] = 1;//
	                        else
	                            eRIC_AES_Key[0] = 0;//encryption since eROS V4.1 and V1.5 eRICEncryption for a start, however it is changed on tx and rx of data
	                        if(CMD[3]!='?' && CMD[4]!='?')      //(count==42) as count is changed this is now based on CMD[3] and CMD[4] before ACK is sent //V1.5 if count is 42ER_CMD#A1yxx..32x
	                        {
	                            char tempkey = 0;
	                            char j = 1;
	                            for(i =0;i<32;i++)	 //V1.5 for taking 32 characters instead of 16
	                            {
	                                //  CMD[3+i] = (CMD[3+i]-0x30); //Covert ascii into real hex value V1.4
	                                CMD[3+i] = eRIC_Ascii2Hex(CMD[3+i]); //Covert ascii into real hex value V1.5
	                                tempkey =  CMD[3+i]<<4;
	                                i++;
	                                CMD[3+i] = eRIC_Ascii2Hex(CMD[3+i]); //Covert ascii into real hex value V1.5
	                                tempkey |= CMD[3+i];
	                                eRIC_AES_Key[j] = tempkey;
	                                j++;
	                            }
	                        }
	                        eRIC_AES_ChangeKey(); //Changed for V4.1 and V1.5
	                    }
	                }
	                else if(CMD[2] == '?')//Read encryption/decryption key
	                {
	                    eRIC_UartASendByte(CurrentAESHasEnabled+'0'); //THis send if AES enabled or only encrypted mode or only decrytpedmode (0 for off,1 for both encryption decryption on,2 for encryption only,3 for decryption only)
	                    while(eRIC_UartATxBufferIsBusy());
					}
	                else
	                {
	                    eRIC_UartASendByte('E');
	                    while(eRIC_UartATxBufferIsBusy());
	                    eRIC_UartASendByte('R');
					    while(eRIC_UartATxBufferIsBusy());
					    eRIC_UartASendByte('R');
					    while(eRIC_UartATxBufferIsBusy());
					    eRIC_UartASendByte('O');
					    while(eRIC_UartATxBufferIsBusy());
					    eRIC_UartASendByte('R');
					    while(eRIC_UartATxBufferIsBusy());
					}
	            }//End of Encryption/Decryption command
	            else if(CMD[1]=='2')// Low power modes command
	            {
	                volatile unsigned char level = eRIC_Ascii2Hex(CMD[2]);  //Added and modified in V1.5.5
	                if(level>0 && level <4)
	                {
	                    CurrentLPMLevel = eRIC_Eeprom_Read(18);
	                    if(CurrentLPMLevel!=level)   //If LPM is not already set.
	                    {
	                        if(eRIC_Eeprom_Read(31))   //CHeck if A8x command is already set to any 1,2,3. IF it is then make it to zero, as A2x and A8x both cannot be enabled.
	                        {
	                            eRIC_Eeprom_Write(31,0);
	                        }
	                        if(eRIC_RxPowerLevel>0 && eRIC_RxPowerLevel<8) //Modified as below so that eeprom is not written in D1-7 1.5.4
	                        {
	                            eRIC_RxPowerLevel = 0;
	                            eRIC_RadioUpdate();
	                            eRIC_Eeprom_Write(18,level); //Write EEprom address 18 which is LPM address
	                            eRIC_RxPowerLevel = CurrentRxPowerLevel;
	                            eRIC_RadioUpdate();
	                        }
	                        else
	                            eRIC_Eeprom_Write(18,level); //Write EEprom address 18 which is LPM address
	                    }
	                    CurrentLPMLevel = 0;
	                }
	                else if(CMD[2]=='?')
				    {
	                    eRIC_UartASendByte(eRIC_Eeprom_Read(18)+'0'); //V1.5.1 THis send if AES enabled or only encrypted mode or only decrytpedmode (0 for off,1 for both encryption decryption on,2 for encryption only,3 for decryption only)

				     //   eRIC_UartASendByte(CurrentLPMLevel+'0'); //THis send if AES enabled or only encrypted mode or only decrytpedmode (0 for off,1 for both encryption decryption on,2 for encryption only,3 for decryption only)
				        while(eRIC_UartATxBufferIsBusy());
				    }
				    else
				    {
				        CurrentLPMLevel = 0;
				        eRIC_Eeprom_Write(18,CurrentLPMLevel); //Write EEprom address 18 which is LPM address V1.5.1
				    }
				}//End of LOW power modes command
	            else if(CMD[1]=='5') //Handshaking command
	            {
	                if(CMD[2]=='?')
	                {
	                    eRIC_UartASendByte(CurrentHandshaking+'0');
	                    while(eRIC_UartATxBufferIsBusy());
	                }
	                else
	                {
	                CurrentHandshaking = eRIC_Ascii2Hex(CMD[2]);
	                eRIC_Eeprom_Write(30,CurrentHandshaking);
	                if(CurrentHandshaking == 1)  //Added V1.5
	                {
	                    Pin2_CTS();
	                    Pin5_RTS();
	                }
	                }

	            }//End of handshaking command
	            else if(CMD[1]=='8') //Only CPU LPM command
	            {
	                volatile unsigned char level = eRIC_Ascii2Hex(CMD[2]);  //Added and modified in V1.5.5.
	                if(level>0 && level <4)
	                {
	                    CurrentCPULPMLevel = eRIC_Eeprom_Read(31);
	                    if(CurrentCPULPMLevel!=level)               //If CPU LPM not already set
	                    {
	                        if(eRIC_Eeprom_Read(18))   //CHeck if A2x command is already set to any 1,2,3. IF it is then make it to zero, as A2x and A8x both cannot be enabled.
	                        {
	                            eRIC_Eeprom_Write(18,0);
	                        }
	                        if(eRIC_RxPowerLevel>0 && eRIC_RxPowerLevel<8) //Modified as below so that eeprom is not written in D1-7 1.5.4
	                        {
	                            eRIC_RxPowerLevel = 0;
	                            eRIC_RadioUpdate();
	                            eRIC_Eeprom_Write(31,level); //Write EEprom address 31 which is CPULPM address
	                            eRIC_RxPowerLevel = CurrentRxPowerLevel;
	                            eRIC_RadioUpdate();
	                        }
	                        else
	                            eRIC_Eeprom_Write(31,level); //Write EEprom address 31 which is CPULPM address
	                    }
	                    CurrentCPULPMLevel = 0;

	                }
	                else if(CMD[2]=='?')
	                {
                        eRIC_UartASendByte(eRIC_Eeprom_Read(31)+'0');  //V1.5.1
	                   // eRIC_UartASendByte(CurrentCPULPMLevel+'0');
	                    while(eRIC_UartATxBufferIsBusy());
	                }
	                else
	                {
	                    CurrentCPULPMLevel = 0;
	                    eRIC_Eeprom_Write(31,CurrentCPULPMLevel); //Write EEprom address 31 which is CPULPM address V1.5.1
	                }
	            }//End of Only CPU LPM command
			}
	        break;
	        case 'B':           // Bandwidth is now data rate (Over air BAUD rate)
	            switch(CMD[1]-0x30)
	            {
	            case 0:
	                eRIC_RfBaudRate = 1200;
	                break;
	            case 1:
	                eRIC_RfBaudRate = 2400;
					break;
				case 2:
					eRIC_RfBaudRate = 4800;
					break;
				case 3:
					eRIC_RfBaudRate = 9600;
					break;
				case 4:
					eRIC_RfBaudRate = 19200;
					break;
				case 5:
					eRIC_RfBaudRate = 38400;
					break;
				case 6:
					eRIC_RfBaudRate = 76800;
					break;
				case 7:
					eRIC_RfBaudRate = 100000;
					break;
				case 8:
					eRIC_RfBaudRate = 250000;
					break;
				case 9:
					eRIC_RfBaudRate = 500000;
					break;
				default:
					eRIC_RfBaudRate = 38400;
					break;
				}
				CurrentDataRate = CMD[1]-'0';
				eRIC_Eeprom_Write(1,CurrentDataRate);
				break;
	        case 'C':  //Command to change Channel
	            eRIC_Channel = CMD[1] - '0';
	            CurrentChannel = CMD[1]-'0';
	            eRIC_Eeprom_Write(2,CurrentChannel);
	            break;
	        case 'D':  //Command to change Rxpowerlevel
	           // eRIC_RxPowerLevel = CMD[1]-'0';
	            CurrentRxPowerLevel = CMD[1]-'0';
	           // eRIC_Eeprom_Write(5,CurrentRxPowerLevel);
	            if(eRIC_RxPowerLevel>0 && eRIC_RxPowerLevel<8) //Modified as below so that eeprom is not written in D1-7 1.5.4
	            {
	                eRIC_RxPowerLevel = CMD[1]-'0';
	            if(eRIC_RxPowerLevel==0)
	            {
	                eRIC_RadioUpdate();
	                eRIC_Eeprom_Write(5,CurrentRxPowerLevel);
	            }
	            }
	            else
	            {
	                eRIC_Eeprom_Write(5,CurrentRxPowerLevel);
	                eRIC_RxPowerLevel = CMD[1]-'0';
	            }
	            break;
	        case 'd':  //Command to change Txpowerlevel
	            eRIC_TxPowerLevel = CMD[1]-'0';
	            CurrentTxPowerLevel = CMD[1]-'0';
	            eRIC_Eeprom_Write(6,CurrentTxPowerLevel);
	            break;
	        case 'f':           //Command to change and read CPUClockfrequency Added in V1.5
	            if(CMD[1]=='?')  //Reads CPUfrequency
	            {
	                volatile signed char i;
	                for (i = 28;i>=0;i=i-4)
	                {
	                    if(((CurrentCpuFrequency>>i)&0xF)<10)
	                        eRIC_UartASendByte((((CurrentCpuFrequency>>i)&0xF)+'0'));
	                    else
	                        eRIC_UartASendByte((((CurrentCpuFrequency>>i)&0xF)+'7'));
	                    while(eRIC_UartATxBufferIsBusy());
	                }
	            }
	            else//Write CPUfrequency
	            {
	                eRIC_TimerA0_InterruptFlag_clear();
	                eRIC_TimerA0_Stop();
	                eRIC_TimerA0_Capture0orCompare0InterruptDisable();
	                eRIC_UartARxInteruptDisable();
	                eRIC_UartATxInteruptDisable();
	                volatile char i = 0, address = 14;
	                while(i<8)
	                {
	                    Tempfrequency <<= 4;
	                    Tempfrequency &= ~0xf;
	                    Tempfrequency += eRIC_Ascii2Hex(CMD[++i]);
	                    Tempfrequency <<= 4;
	                    Tempfrequency &= ~0xf;
	                    Tempfrequency += eRIC_Ascii2Hex(CMD[++i]);
	                    while(eRIC_UartATxBufferIsBusy());
	                    eRIC_Eeprom_Write(address++,(unsigned char)Tempfrequency);
	                }
	             //   eRIC_CpuFrequency = Tempfrequency;
	                CurrentCpuFrequency = Tempfrequency;
	                if(eRIC_CpuFrequency!=CurrentCpuFrequency)
	                    eRIC_SetCpuFrequency(CurrentCpuFrequency);
	                if(CurrentCpuFrequency<1000000)
	                {
	                    Bool_INLPM_LOWCPU = 1;//V1.5.5 THis is set when LPM MODES are set and PIn22 is low or CPU is <1000000
	                }
	                else
	                {
	                    Bool_INLPM_LOWCPU = 0;//V1.5.5 THis is set when LPM MODES are set and PIn22 is low or CPU is <1000000
	                }
	                eRIC_UARTAInitialise(BaudValues[CurrentUartBaud]);
	                eRIC_UartARxInteruptEnable();
	                SetupTimer();
	                SetLedtimerTimeOut();
	                SetUARTTimeout();
	            }
	        break;
	        case 'F':           //Command to change and read frequency
	            if(CMD[1]=='?')  //Reads frequency
	            {
	                volatile signed char i;
	                for (i = 28;i>=0;i=i-4)
	                {
	                    if(((CurrentFrequency>>i)&0xF)<10)
	                        eRIC_UartASendByte((((CurrentFrequency>>i)&0xF)+'0'));
	                    else
	                        eRIC_UartASendByte((((CurrentFrequency>>i)&0xF)+'7'));
	                    while(eRIC_UartATxBufferIsBusy());
	                }
	            }
	            else//Write frequency
	            {
	                volatile char i = 0, address = 7;
	                while(i<8)
	                {
	                    Tempfrequency <<= 4;
	                    Tempfrequency &= ~0xf;
	                    Tempfrequency += eRIC_Ascii2Hex(CMD[++i]);
	                    Tempfrequency <<= 4;
	                    Tempfrequency &= ~0xf;
	                    Tempfrequency += eRIC_Ascii2Hex(CMD[++i]);
	                    while(eRIC_UartATxBufferIsBusy());
	                    eRIC_Eeprom_Write(address++,(unsigned char)Tempfrequency);
	                }
	                eRIC_Frequency = Tempfrequency;
	                CurrentFrequency = eRIC_Frequency;
	                Power(); //V1.5
	                eRIC_Eeprom_Write(3,CurrentPowerLevel);
	            }
	        break;
	        case 'L':           //  Currently only used to get serial number, to read and write eeprom and to read and write GroupIDs
	            switch(CMD[1]-0x30)
	            {
	            case 4://Command for Eeprom
	            {
	                unsigned char Address = (unsigned char)((eRIC_Ascii2Hex(CMD[2])<<4) | (eRIC_Ascii2Hex(CMD[3])));
	                Address &= 0xFF;
	                unsigned char data = 0;
	                if(CMD[4]=='?')  // Read the EEPROM Location
	                {
	                    data = eRIC_Eeprom_Read(Address);
	                    eRIC_UartASendByte(eRIC_Int2Ascii((data>>4)& 0xf));
	                    while(eRIC_UartATxBufferIsBusy());
	                    eRIC_UartASendByte(eRIC_Int2Ascii(data & 0xf));
	                }
	                else //Write into eeprom
	                {
	                    data = (unsigned char)((eRIC_Ascii2Hex(CMD[4])<<4) | (eRIC_Ascii2Hex(CMD[5])));
	                    eRIC_Eeprom_Write(Address,data);
	                    if(Address>19 && Address<30) //if power location is changed,,update power value
	                    {
	                        PowerValue[Address-20] = data;
	                        if(CurrentPowerLevel==(Address-20)) //change radio power only when currentpowerlevel is changed in EEprom location
	                            Power();
	                    }
	                }
	            break;
	            }
	            case 7://Command to read and write GroupIDs
	                if(CMD[2]=='?')  //To read GroupID
	                {
	                    volatile signed char i=0;
	                    for (i = 12;i>=0;i=i-4)//Gets 2 bytes if group id
	                    {
	                        if(((eRIC_GroupID>>i)&0xF)<10)
	                            eRIC_UartASendByte((((eRIC_GroupID>>i)&0xF)+'0'));
	                        else
	                            eRIC_UartASendByte((((eRIC_GroupID>>i)&0xF)+'7'));
	                        while(eRIC_UartATxBufferIsBusy());
	                    }
	                }
	                else //To write GroupIDs
	                {
	                    volatile unsigned int temp = eRIC_Ascii2Hex(CMD[2]);
	                    temp <<=4;
	                    temp |= eRIC_Ascii2Hex(CMD[3]);
	                    temp <<=4;
	                    temp |= eRIC_Ascii2Hex(CMD[4]);
	                    temp <<=4;
	                    temp |= eRIC_Ascii2Hex(CMD[5]);
	                    eRIC_GroupID = temp;   //Needs radio update which is at the end
	                    CurrentGroupID = eRIC_GroupID;
	                    eRIC_Eeprom_Write(11,(CurrentGroupID>>8));
	                    eRIC_Eeprom_Write(12,(CurrentGroupID));
	                    if(eRIC_GroupID)//IF groupID is 0000, the groupID is disabled ,,only in companion app
	                    {
	                        eRIC_GroupIDEnable(eRIC_GroupID);
	                    }
	                    else
	                    {
	                        eRIC_GroupIDDisable();
	                    }
	                }
	            break;
	            case 8://Command For serial number
	                if(CMD[2]=='?')  //Gets serial number
	                {
	                    volatile signed char i=0;
	                    for (i = 28;i>=0;i=i-4)//Gets 4 bytes if serial number
	                    {
	                        if(((eRIC_GetSerialNumber()>>i)&0xF)<10)
	                            eRIC_UartASendByte((((eRIC_GetSerialNumber()>>i)&0xF)+'0'));
	                        else
	                            eRIC_UartASendByte((((eRIC_GetSerialNumber()>>i)&0xF)+'7'));
	                        while(eRIC_UartATxBufferIsBusy());
	                    }
	                }
	                break;
	            default:
	                break;
	            }
	            break;
	        case 'U'://Command for UartBaud rate
	            eRIC_UartARxInteruptDisable(); //V1.5
	            eRIC_UartATxInteruptDisable(); //V1.5
	            CurrentUartBaud = CMD[1]-'0';
	            eRIC_UARTAInitialise(BaudValues[CurrentUartBaud]); //V1.5
	            SetUARTTimeout();
	            eRIC_UartARxInteruptEnable(); //V1.5
	            eRIC_Eeprom_Write(4,CurrentUartBaud);
	            break;
	        case 'P'://Command to change Power
	            CurrentPowerLevel = CMD[1]-'0';
	            Power();//V1.5
	            eRIC_Eeprom_Write(3,CurrentPowerLevel);
	            break;
	        case 'R': //Command to reset the module, resets radio to default settings
	            switch(CMD[1])
	            {
	            case '0':  //Resets radio to default state
	                if(eRIC_RxPowerLevel>0 && eRIC_RxPowerLevel<8) //Modified as below so that eeprom is not written in D1-7 1.5.4
	                {
	                    eRIC_RxPowerLevel = 0;
	                    eRIC_RadioUpdate();
	                }
	                eRIC_Eeprom_Write(0,0);//Change eeprom(0) to 0, so that it will load default settings when called radiostoredsettings
	               /* Radio_Stored_settings();
	                if(CurrentCarrierMode>0 && CurrentCarrierMode<4) //V1.5
	                {
	                    SetCarrierModeOnStartup(CurrentCarrierMode);
	                }
	                SetupTimer();
	                SetLedtimerTimeOut(); //V1.5
	                break;*/ //removed in V1.5.5. After writing EEprom location 0 with 0 and doing a Power on reset will restore all setting to default
	            case '1':  //POR power on reset    //V1.5
	                //eRIC_PowerOnReset(); //Resets the radio
	                eRIC_BrownOutReset();//Resets the Radio. POR has issues of reseting again when channel command or any other command is sent
	                break;
	            case '2':
	                eRIC_FlashProgram_Mode(0); // 0 = Flash VIA UART on pins 3 & 4
	                break;
	            case '?': //Cause of last reset can be retrieved using this variable. Refer Page 78 of slau259e.pdf //V1.5.5
	            {
	                volatile signed char i;
	                for (i = 12;i>=0;i=i-4)
	                {
	                    if(((CurrentCpuFrequency>>i)&0xF)<10)
	                        eRIC_UartASendByte((((eRIC_LastCauseOfReset>>i)&0xF)+'0'));
	                    else
	                        eRIC_UartASendByte((((eRIC_LastCauseOfReset>>i)&0xF)+'7'));
	                    while(eRIC_UartATxBufferIsBusy());
	                }
	                break;
	            }
	            default:
	                break;
	            }
	            break;
	        case 'T'://All test commands
	        {
	            switch(CMD[1])
	            {
	            case '0':
	                CurrentCarrierMode = 4;
	                eRIC_SetHighSideCarrierOn();
	                break;
	            case '1':
	                CurrentCarrierMode = 4;
	                if(CMD[2]=='U')//Uncalibrated modulation carrier on ,added in V1.4
	                {
	                    buff[9] = '0';  //This is to clear that 'U' in buffer ,otherwise when ER_CMD#T1 is sent U is still present
	                    eRIC_SetUncalibratedModulationCarrierOn();
	                }
	                else
	                {
	                    eRIC_SetModulationCarrierOn();
	                }
	                break;
	            case '2':
	                CurrentCarrierMode = 4;
	                eRIC_SetLowSideCarrierOn();
	                break;
	            case '3':
	                count = 0;
	                while(VersionString[count] != '\n') // wait for terminating null
	                {
	                    while(eRIC_UartATxBufferIsBusy());
	                    if(count ==4)
	                    {
	                        unsigned char *Flash_ptr;
	                        Flash_ptr = (unsigned char *)(0x1800);// This is the address of serial number , where 9 or 4 is loaded
	                        eRIC_UartASendByte(((*Flash_ptr>>4)+'0'));//0x90 or 0x40..where 9 or 4 is sent
	                        while(eRIC_UartATxBufferIsBusy());
	                    }
	                    eRIC_UartASendByte(VersionString[count++]);//teststring[ii++];
	                }
	                break;
	            case '4':
	                eRIC_RadioAsyncMode();
	                Pin2_SetAsAsyncRxData();
	                eRIC_Rx_Enable();
	                break;
	            case '5':
	                eRIC_RadioPacketMode();
	                Pin2_FunctionNone();   //V1.5
	                Pin2_FunctionIO();    //V1.5
	                Pin2_CTS();           //V1.5
	                if(CurrentHandshaking == 1)  //Added V1.5
	                    Pin5_RTS();
	                break;
	            case '6':
	                CurrentCarrierMode = eRIC_Eeprom_Read(19);//V1.5 Read from 19 location which is Z command
	                eRIC_SetCarrierOff();
	                break;
	            case '7':
	            {
	                char buff[10];
	                memset(buff,0,10);
	                volatile int i;
	                int length = eRIC_Sprintf(buff,"%fC",eRIC_GetTemperature());
	                for(i = 0; i < length; i++)
	                {
	                    while(eRIC_UartATxBufferIsBusy());
	                    eRIC_UartASendByte(buff[i]);
	                }
	                break;
	            }
	            case '8':
	            {
	                char buff[10];
	                volatile int i;
	                signed char db;
	                db = eRIC_GetLastPacketRSSI(); //Gets last packet RSSI Value
	                int length = eRIC_Sprintf(buff,"%ddBm",(float)db);
	                for(i = 0; i < length; i++)
	                {
	                    while(eRIC_UartATxBufferIsBusy());
	                    eRIC_UartASendByte(buff[i]);
	                }
	                break;
	            }
	            case '9':
	            {
	                char buff[10];
	                volatile int i;
	                signed char db;
	                db = eRIC_GetLiveRSSI(); //Gets real RSSI Value with sign
	                int length = eRIC_Sprintf(buff,"%ddBm",(float)db);
	                for(i = 0; i < length; i++)
	                {
	                    while(eRIC_UartATxBufferIsBusy());
	                    eRIC_UartASendByte(buff[i]);
	                }
	                break;
	            }
	            default:
	                break;
	            }
	            return 1;
	        }
	        case 'Z':  //V1.5
	            if(CMD[1]=='0')//Command to set the module in any one carrier setting on startup
	            {
	                if(CMD[2]=='?')
	                {
	                    eRIC_UartASendByte(CurrentCarrierMode+'0');
	                    while(eRIC_UartATxBufferIsBusy());
	                }
	                else
	                {
	                    CurrentCarrierMode = eRIC_Ascii2Hex(CMD[2]);
	                    eRIC_Eeprom_Write(19,CurrentCarrierMode);
	                    SetCarrierModeOnStartup(CurrentCarrierMode);
	                }
	            }
	            break;
	        default:
	            return 0;
	        }

			eRIC_RadioUpdate();
			return 1;
		}
	} //End of if(WaitingForACK)
	// Test if a command has been sent
	else
	{
		if(count < 9)
			return 0;
		char MaxFunctionValue = '9';
		if(0 == eRIC_Stringcompare("ER_CMD#", buff,7))
		{
			//eRIC_Stringcopy(CMD,&buff[7],9); // Copy the command for when the ACK is sent
			//eRIC_Stringcopy(CMD,&buff[7],20); // V1.4 MOdified from 9 - 20 for AES command Copy the command for when the ACK is sent
			eRIC_Stringcopy(CMD,&buff[7],36); //  V1.5 MOdified from 20 to 36 for AES command Copy the command for when the ACK is sent

			switch(CMD[0])
			{
			case 'a': //Carrier detect V1.5.5
			{
			case 'F': // CPU Frequency control
			{
			case 'f':
			{
				WaitingForACK = 1;
				unsigned char count1 = 0;
				while(count1 < count)
				{
					while(eRIC_UartATxBufferIsBusy());
					eRIC_UartASendByte(buff[count1++]);
				}
				return 1;
			}
			}
			}
			case 'A': // AES Command
			    if(CMD[1]=='1')
			    {
			    if(count>9 && count!=42)
			    {
			        CMD[3] = '?';//based on these , the logic is formed on ER_CMD#A1 command as count is changed later
			        CMD[4] = '?';
			        if(count!=10)
			        {
			        eRIC_UartASendByte('E');
			        while(eRIC_UartATxBufferIsBusy());
			        eRIC_UartASendByte('R');
			        while(eRIC_UartATxBufferIsBusy());
			        eRIC_UartASendByte('R');
			        while(eRIC_UartATxBufferIsBusy());
			        eRIC_UartASendByte('O');
			        while(eRIC_UartATxBufferIsBusy());
			        eRIC_UartASendByte('R');
			        while(eRIC_UartATxBufferIsBusy());
			        return 1;
			        }
			    }
				WaitingForACK = 1;
				unsigned char count1 = 0;
				while(count1 < count)
				{
					while(eRIC_UartATxBufferIsBusy());
					eRIC_UartASendByte(buff[count1++]);
				}
				return 1;
                }
			    else if(CMD[1]=='2' || CMD[1]=='5'|| CMD[1]=='8') //Low power modes || Handshaking || Only CPU LPM
			    {
			        WaitingForACK = 1;
			        unsigned char count1 = 0;
			        while(count1 < count)
			        {
			            while(eRIC_UartATxBufferIsBusy());
			            eRIC_UartASendByte(buff[count1++]);
			        }
			        return 1;
			    }
                else
                    return 0;
			case 'B': // Bandwidth is replaced with data rate
				CurrentVal = CurrentDataRate;
				break;
			case 'C'://Return current channel
				CurrentVal = CurrentChannel;
				break;
			case 'D': //return current Rxpowerlevel
				//MaxFunctionValue = '7'; //V1.4
				MaxFunctionValue = '8'; //V1.5
				CurrentVal = CurrentRxPowerLevel;
				break;
			case 'd': //Returns current Txpowerlevel
			    //MaxFunctionValue = '7'; //V1.4
			    MaxFunctionValue = '8'; //V1.5
				CurrentVal = CurrentTxPowerLevel;
				break;
			case 'U': // Returns uart baudrate
				CurrentVal = CurrentUartBaud;
				break;
			case 'L':
				break;
			case 'P': //Returns current powerlevel
				MaxFunctionValue = '<';
				CurrentVal = CurrentPowerLevel;
				break;
			case 'T':
				break;
			case 'R':
			    WaitingForACK = 1;
			    unsigned char count1 = 0;
			    while(count1 < count)
			    {
			        while(eRIC_UartATxBufferIsBusy());
			        eRIC_UartASendByte(buff[count1++]);
			    }
			    return 1;
				//break;
			case 'Z': //For FCC purpose
			    break;
			default:
				return 0;
			}
			// Test if valid numbers are used:
			if((CMD[1] <= MaxFunctionValue && CMD[1] >= '0') || CMD[1] == '?' )
			{
				volatile unsigned char count1 = 0;
				if(CMD[1] == '?')
				{
					buff[8] = (CurrentVal+'0');
				}
				else
				{
					WaitingForACK = 1;
				}
				while(count1 < count)
				{
					while(eRIC_UartATxBufferIsBusy());
					eRIC_UartASendByte(buff[count1++]);
				}
				return 1;
			}
		}
	}
	return 0;
}



void SetupTimer() //V1.5 Timer A0 is replaced using eRIC.c and header definitions
{
    eRIC_TimerA0_Setup(eRICTimer_Cs_CPU+eRICTimer_Div_1+eRICTimer_UpMode+eRICTimer_Reset);

    eRIC_TimerA0_Capture0orCompare0_Data = 100; //100 clock cycles then interrupt

    eRIC_TimerA0_Capture0orCompare0Setup(eRICTimer_CompareMode+eRICTimer_CCInterruptEnable+eRICTimer_OutputMode_OutputOnly);
}

/*
 * THis is just for Blinking LED1 every second for any clock speed
 */
void SetLedtimerTimeOut()  //V1.5
{
    LED_Timer = 1;
    LED_Timer_Timeout_1s = eRIC_CpuFrequency/100;//101 //V1.5

    if(LED_Timer_Timeout_1s<1024)  //added in V1.5.5
        LED_Timer_Timeout_1ms = 1;          //added in V1.5.5
    else
        LED_Timer_Timeout_1ms = LED_Timer_Timeout_1s/1024; //added in V1.5.5

}

#pragma vector=TIMER0_A0_VECTOR
__interrupt void TIMER0__A0_ISR(void) // set for every 100Cycles
{

	Uart_buff_Timeout++;    // just count intervals

	if(CommandTimeout)
	{
	    CommandTimeout--;  //Added in V1.5.5
	    if(CommandTimeout==0)   //If command sent/received is timedout and then if there is any Rf data available to read then set flag so that it is dealt other wise it wont be read untill next byte is received over air
	    {
	        if(eRIC_Rxdata_available)
	            eRIC_RfDataReceivedInterruptFlag_Set();
	    }
	}



	if(--LED_Timer == 0)
	{
		if(Pin16_Read())
		{
			Pin16_SetLow();
			LED_Timer = LED_Timer_Timeout_1s;//V1.5
		}
		else
		{
			Pin16_SetHigh();
			LED_Timer = LED_Timer_Timeout_1ms;//added in V1.5.5
		}
	}
}

// UART Interrupt Vector
#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR(void)
{
    volatile unsigned int function = UCA0IV;             // Read Interrupt Vector triggered
    switch(function)
    {
    case USCI_NONE:                                              // Vector 0 - no interrupt
        break;
    case USCI_UCRXIFG:                   // Vector 2 - RXIFG UART RECEIVE
        UART_RX_Buffer[UART_RX_Count++] = eRIC_UartAReceiveByte() ; //Added in here in V1.5.5 from below
        eRIC_RfDataReceivedInterruptDisable();
        if(Bool_INLPM_LOWCPU)//V1.5.5 THis is set when LPM MODES are set and PIn22 is low or CPU is <1000000
        {
            Uart_buff_Timeout = Uart_Timeout;

            while((Uart_buff_Timeout--))//549 //V1.5.4   //Point A from here to point B down it takes 28 cycles roughly, so based on this the time out dividing factor is calculated in SetUARTTimeout
            {
                do  //Added V1.5.4
                {
                    if(eRIC_UartARxHasInterrupted())//(UCA0IFG & UCRXIFG) // Has data started?
                    {
                        // eRIC_UartARxClearInterruptFlag();//V1.5.4
                        UART_RX_Buffer[UART_RX_Count++] = eRIC_UartAReceiveByte();
                        Uart_buff_Timeout = Uart_Timeout;// Reset counter
                    }
                }while(eRIC_UartABusy());
            }      //Point B
            Uart_buff_Timeout = Uart_Timeout+1;//Added in V1.5.5 this is because in EasyRadio_Transmit_routin() it checks if(Uart_buff_Timeout > Uart_Timeout), generaaly timer will increment Uart_buff_Timeout, but incase if it is turnedd of in LPM modes it wont increment and thats why this is needed.
            EasyRadio_Transmit_routine();
            return;
            }
            Uart_buff_Timeout = 0;
        break;
    case USCI_UCTXIFG:                   // Vector 4 - TXIFG
        if(eRIC_Rxdata_available)
        {
            eRIC_UartASendByte(eRIC_ReadRfByte());
        }
        else
        {
            eRIC_UartATxInteruptDisable();
        }
        break;
    default:
        break;
    }
}

/*THis function is added in V1.5. THis is called by command ER_CMD#A2x . It will set three LPM modes 1,2,3 and will exit LPM when Pin22
 * is set high. Before setting LPM, we need to turn off interrupt and enable only interrupts we need, and set the CPU and Rxpowerleve
 * in lowest possible according to level. That is why this function is used, instead of only calling eRIC_LPM_Levelx();
 *
 */

void SetLPM(volatile unsigned char level)
{
    if(!Pin22_Read())
    {
        Bool_INLPM_LOWCPU = 1;//V1.5.5 THis is set when LPM MODES are set and PIn22 is low or CPU is <1000000
        if(level==1)  //Added V1.5
        {
            if(CurrentCPULPMLevel)
            {
                //    CurrentLPMLevel = 1;
                eRIC_TimerA0_InterruptFlag_clear();
                eRIC_TimerA0_Stop();
                eRIC_TimerA0_Capture0orCompare0InterruptDisable();
                SetUARTTimeout();
                Pin22_ClearInterruptFlag();//V1.5.1
                Pin22_InterruptEnable();//V1.5.1
                eRIC_LPM_Level0();
                CurrentCPULPMLevel = 0;
                //At this point CPUlpm level will be 0
                SetUARTTimeout();
                SetupTimer();
            }
            else
            {
                eRIC_TimerA0_InterruptFlag_clear();
                eRIC_TimerA0_Stop();
                eRIC_TimerA0_Capture0orCompare0InterruptDisable();
                eRIC_UartARxInteruptDisable();
                eRIC_UartATxInteruptDisable();
                if(CurrentRxPowerLevel<7)
                {
                    eRIC_RxPowerLevel = CurrentRxPowerLevel = 7;
                    eRIC_RadioUpdate();
                }
                if(CurrentCpuFrequency>1048576)
                {
                    CurrentCpuFrequency = 1048576;
                    eRIC_SetCpuFrequency(1048576);
                }
                if(CurrentUartBaud>4)
                    CurrentUartBaud = 4;
                eRIC_UARTAInitialise(BaudValues[CurrentUartBaud]);
                CurrentLPMLevel = 1;
                SetUARTTimeout();
                eRIC_UartARxInteruptEnable();
  //              eRIC_UartATxInteruptEnable();
                Pin22_ClearInterruptFlag();//V1.5.1
                Pin22_InterruptEnable();//V1.5.1
                eRIC_LPM_Level0();
                CurrentLPMLevel = 0;     //V1.5.4
                //At this point lpm level will be 0
                //     eRIC_Eeprom_Write(18,CurrentLPMLevel); //Write EEprom address 18 which is LPM address
                //    Radio_Stored_settings();
                //Gets CPUCLOCKFrequency value from Eeprom location
                CurrentCpuFrequency = eRIC_Eeprom_Read(14);            //Reads Eeprom location 14 and assigns to current CPU frequency high byte
                CurrentCpuFrequency<<=8;
                CurrentCpuFrequency |= eRIC_Eeprom_Read(15);           //Reads Eeprom location 15 and assigns to current CPU frequency next high byte
                CurrentCpuFrequency<<=8;
                CurrentCpuFrequency |= eRIC_Eeprom_Read(16);           //Reads Eeprom location 16 and assigns to current CPU frequency next high byte
                CurrentCpuFrequency<<=8;
                CurrentCpuFrequency |= eRIC_Eeprom_Read(17);          //Reads Eeprom location 17 and assigns to current CPU frequency low byte
                if(eRIC_CpuFrequency!=CurrentCpuFrequency)
                    eRIC_SetCpuFrequency(CurrentCpuFrequency);
                CurrentUartBaud = eRIC_Eeprom_Read(4);
                eRIC_UARTAInitialise(BaudValues[CurrentUartBaud]);
                eRIC_UartARxInteruptEnable();
                SetUARTTimeout();
                CurrentRxPowerLevel = eRIC_Eeprom_Read(5);          //Reads Eeprom location 5 and assigns to current Rxpowerlevel
                //eRIC_RadioUpdate();
                if(CurrentRxPowerLevel<7)
                {
                    eRIC_RxPowerLevel = CurrentRxPowerLevel;
                    eRIC_RadioUpdate();
                }
                SetLedtimerTimeOut();
                SetupTimer();
            }
        }
        else if(level==2)  //Added V1.5
        {
            eRIC_TimerA0_InterruptFlag_clear();
            eRIC_TimerA0_Stop();
            eRIC_TimerA0_Capture0orCompare0InterruptDisable();

            if(CurrentCPULPMLevel)
            {
                SetUARTTimeout();
                Pin22_ClearInterruptFlag();//V1.5.1
                Pin22_InterruptEnable();//V1.5.1

                eRIC_LPM_Level1();
                CurrentCPULPMLevel = 0;
                //At this point CPUlpm level will be 0
                SetUARTTimeout();
                SetupTimer();
            }
            else
            {
                eRIC_UartARxInteruptDisable();
                eRIC_UartATxInteruptDisable();
                eRIC_RfDataReceivedInterruptDisable(); //V1.5.5
                 if(CurrentRxPowerLevel<7)
                {
                    eRIC_RxPowerLevel = CurrentRxPowerLevel = 7;
                    eRIC_RadioUpdate();
                }
                 if(CurrentCpuFrequency>32768)
                {
                    CurrentCpuFrequency = 32768;
                    eRIC_SetCpuFrequency(32768);
                }
                 CurrentUartBaud = 0;
                eRIC_UARTAInitialise(1200);
                CurrentLPMLevel = 2;
                SetUARTTimeout();
                eRIC_UartARxInteruptEnable();
                Pin22_ClearInterruptFlag();//V1.5.1
                Pin22_InterruptEnable();//V1.5.1

                eRIC_RfDataReceivedInterruptEnable(); //V1.5.5
                eRIC_LPM_Level1();
                CurrentLPMLevel = 0; //V1.5.4
                //At this point lpm level will be 0
                //Gets CPUCLOCKFrequency value from Eeprom location
                CurrentCpuFrequency = eRIC_Eeprom_Read(14);            //Reads Eeprom location 14 and assigns to current CPU frequency high byte
                CurrentCpuFrequency<<=8;
                CurrentCpuFrequency |= eRIC_Eeprom_Read(15);           //Reads Eeprom location 15 and assigns to current CPU frequency next high byte
                CurrentCpuFrequency<<=8;
                CurrentCpuFrequency |= eRIC_Eeprom_Read(16);           //Reads Eeprom location 16 and assigns to current CPU frequency next high byte
                CurrentCpuFrequency<<=8;
                CurrentCpuFrequency |= eRIC_Eeprom_Read(17);          //Reads Eeprom location 17 and assigns to current CPU frequency low byte
                if(eRIC_CpuFrequency!=CurrentCpuFrequency)
                    eRIC_SetCpuFrequency(CurrentCpuFrequency);
                CurrentUartBaud = eRIC_Eeprom_Read(4);
                eRIC_UARTAInitialise(BaudValues[CurrentUartBaud]);
                eRIC_UartARxInteruptEnable();
                CurrentRxPowerLevel = eRIC_Eeprom_Read(5);          //Reads Eeprom location 5 and assigns to current Rxpowerlevel

                if(CurrentRxPowerLevel<7)
                {
                    eRIC_RxPowerLevel = CurrentRxPowerLevel;
                    eRIC_RadioUpdate();
                }
                //eRIC_RadioUpdate();
                SetUARTTimeout();
                SetupTimer();
            }
        }
        else if(level==3)  //Added V1.5
        {
            eRIC_Rx_Disable();
            if(!CurrentCPULPMLevel)
                CurrentLPMLevel = 3;
            Pin22_ClearInterruptFlag();//V1.5.1
            Pin22_InterruptEnable();//V1.5.1
            eRIC_LPM_Level2();
            CurrentLPMLevel = 0;         //V1.5.4
            CurrentCPULPMLevel = 0;
            //At this point lpm level will be 0
            if(CurrentRxPowerLevel<8)
            {
                eRIC_RxPowerLevel = CurrentRxPowerLevel;
                eRIC_RadioUpdate();
            }
        }
    }//end if Pin22read
    else
    {
        CurrentLPMLevel = 0;         //V1.5.4
        CurrentCPULPMLevel = 0;
    }
    if(CurrentCpuFrequency>999999)
        Bool_INLPM_LOWCPU = 0;//V1.5.5 THis is set when LPM MODES are set and PIn22 is low or CPU is <1000000
}

/*THis is added in V1.5. THis is used for FCC testing purpose only. When the ER_CMD#Zxx command is used , it will set eric in one
 * of the carrier mode and also stores the setting in EEPROm. So when startup, eric starts on same carried mode, it was set before.
 */
void SetCarrierModeOnStartup(unsigned char Mode)
{
    if(Mode == 1)
    {
        eRIC_SetHighSideCarrierOn();
    }
    else if(Mode == 2)
    {
        eRIC_SetModulationCarrierOn();
    }
    else if(Mode == 3)
    {
        eRIC_SetLowSideCarrierOn();
    }
    else
        eRIC_SetCarrierOff();
}

/*Power function is added in V1.5. THis is mainly used to assign dbm value in to eRIC_Power before calling Update.
 * THis is also mainly used for FCC version module which can limit power for 869 and 915Mhz
 */
void Power()
{
    eRIC_Power = PowerValue[CurrentPowerLevel];
#ifdef eRIC9
#ifdef limitOnPower_FCC   //V1.5
    signed char temp = 0;
    if(CurrentFrequency>=869750000 && CurrentFrequency<=870000000)
    {
        temp = PowerValue[CurrentPowerLevel];
        while((CurrentPowerLevel-temp)!=2)
        {
            if((CurrentPowerLevel-temp)>2)
                temp++;
            else
                temp--;
        }
        eRIC_Power = temp;
    }
    else if(CurrentFrequency>=902000000 && CurrentFrequency<=928000000)
    {
        temp = PowerValue[CurrentPowerLevel];
        while((CurrentPowerLevel-temp)!=12)
        {
            if((CurrentPowerLevel-temp)>12)
                temp++;
            else
                temp--;
        }
        eRIC_Power = temp;
    }
#endif
#endif
}
